const parser = require("@babel/parser");
const { transformFromAstSync } = require("@babel/core");

const importPlugin = require("./plugins/importPlugin");

const fs = require("fs");
const path = require("path");

fs.readFile(path.resolve(__dirname, "source/c1.js"), "utf8", (err, source) => {
	if (err) {
		console.error(err);
		return;
	}

	/**
	 * sourceType
	 * 指定是否支持解析模块语法，有 module、script、unambiguous 3个取值
	 * module 是解析 es module 语法
	 * script 则不解析 es module 语法，当作脚本执行
	 * unambiguous 则是根据内容是否有 import 和 export 来确定是否解析 es module 语法
	 */
	const ast = parser.parse(source, {
		sourceType: "unambiguous",
		plugins: ["jsx"],
	});

	const { code } = transformFromAstSync(ast, source, {
		plugins: [
			[
				importPlugin,
				{
					components: ["@xiaoe/spark", "@xiaoe/sense", "@xiaoe/material_center_box"],
				},
			],
		],
	});
    
	fs.writeFileSync(path.resolve(__dirname, "dist/index.js"), code);
});
