const { declare } = require("@babel/helper-plugin-utils");
const t = require("@babel/types");

const importPlugin = declare((api, options, dirname) => {
	const astList = [];
	let variable = [];
	return {
		visitor: {
			Program: {
				enter(path, state) {
					path.traverse({
						ImportDeclaration: {
							enter(curPath) {
								const requirePath = curPath.get("source").node.value;
								if (options.components.includes(requirePath)) {
									const specifiers = curPath.get("specifiers");
									const sourceName = requirePath.replace("@xiaoe/", "");
									specifiers.map((spec, idx) => {
										const { local: localNode, imported: importedNode } = spec.node;
										const localName = localNode.name;
										// import Spark from "@xiaoe/spark";
										if (spec.isImportDefaultSpecifier()) {
											!variable.includes(localName) &&
												astList.push(
													api.template(`
											let %%localName%% = window.xiaoe.%%sourceName%%;
											if(!%%localName%%){
												%%localName%% = require(%%requirePath%%).default
											}
										`)({ sourceName, localName, requirePath: t.stringLiteral(requirePath) })
												);
											variable.push(localName);
										}
										// import { Loading as XiaoeLoading, Dialog } from "@xiaoe/sense";
										else if (spec.isImportSpecifier()) {
											!variable.includes(sourceName) &&
												astList.push(
													api.template(`
												let %%sourceName%% = window.xiaoe.%%sourceName%%;
												if(!%%sourceName%%) {
													%%sourceName%% = require(%%requirePath%%).default
												}
												`)({ sourceName, requirePath: t.stringLiteral(requirePath) })
												);
											variable.push(sourceName);
											if (importedNode && importedNode.name !== localName) {
												astList.push(
													api.template(`const %%localName%% = %%sourceName%%.%%importedName%%;
												`)({ sourceName, localName, importedName: importedNode.name })
												);
											} else {
												astList.push(
													api.template(`
														const %%localName%% = %%sourceName%%.%%localName%%;
												`)({ sourceName, localName })
												);
											}
										}
										// import * as MaterialCenterBox from "@xiaoe/material_center_box"
										else if (spec.isImportNamespaceSpecifier()) {
											astList.push(
												api.template(`
											let %%localName%% = window.xiaoe.%%sourceName%%;
											if(!%%localName%%){
												%%localName%% = require(%%requirePath%%).default
											}
										`)({ sourceName, localName: localNode.name, requirePath: t.stringLiteral(requirePath) })
											);
										}
									});
									path.stop();
								}
							},
							exit(curPath) {
								const requirePath = curPath.get("source").node.value;
								if (astList.length) {
									for (let i = 0; i < astList.length; i++) {
										curPath.insertBefore(astList.shift());
									}
									options.components.includes(requirePath) && curPath.remove();
								}
							},
						},
					});
				},
			},
		},
		post(){
			variable = []
		}
	};
});

module.exports = importPlugin;
